package com.ho.test.aop.cache.MAINTEST;

import java.util.Calendar;

import junit.framework.TestCase;

import org.springframework.context.ApplicationContext;
import org.springframework.context.support.FileSystemXmlApplicationContext;

import com.ho.test.aop.cache.service.IServiceToCache;
import com.ho.test.aop.cache.service.common.CacheInterceptor;

/**
 * <p>Title: CacheInterceptorTest</p>
 * <p>Copyright: Copyright (c) 2010</p>
 * <p>Company: HUO</p>
 * 
 * <p>
 * Check if the CacheInterceptor works.
 * </p>
 * 
 * @author Huseyin OZVEREN
 */
public class CacheInterceptorTest extends TestCase {
	
	// -------------------------------------------------------------- CONSTANTS
	// ----------------------------------------------------- PRIVATE ATTRIBUTES
	
	/**
	 * Contains a reference to the spring context.
	 */
	private static ApplicationContext ctx = null;

	// ------------------------------------------------------------- EXCEPTIONS
	// ------------------------------------------------------- INTERNAL CLASSES
	// ----------------------------------------------------------- CONSTRUCTORS
	// --------------------------------------------------------- PUBLIC METHODS
	
	public void testCache() {
		
		IServiceToCache service = (IServiceToCache) ctx.getBean("serviceToCache");
		String output1, output2;
		
		// Call the method1ToCache, this should take 2 seconds...
		// verify if the call to the service takes 2 seconds,
		// if not, fail
		{
			Calendar begin = Calendar.getInstance();
			output1 = service.method1ToCache();
			System.out.println(output1);
			Calendar end = Calendar.getInstance();
			long time = end.getTimeInMillis()-begin.getTimeInMillis();
			if (time < 2000) {
				fail("the cache was active...");
			} // end-if
			
		} // end-if
		
		// Call the method1ToCache again
		// verify if the call to the service does NOT take 2 seconds,
		// if not, fail
		{
			Calendar begin = Calendar.getInstance();
			output2 = service.method1ToCache();
			System.out.println(output2);
			Calendar end = Calendar.getInstance();
			long time = end.getTimeInMillis()-begin.getTimeInMillis();
			if (time >= 2000) {
				fail("the cache was NOT active...");
			} // end-if
		} // end-if
		
		assertTrue(output1.equals(output2));
	}

	
	public void testNoCache() {
		
		IServiceToCache service = (IServiceToCache) ctx.getBean("serviceToCache");
		String output1, output2;
		
		// Call the method2, this method id not in cache...
		// verify if the call to the service takes a different return,
		// if not, fail
		{
			output1 = service.method2();
			System.out.println(output1);
		} // end-if
		
		// Call the method2 again
		// verify if the call to the service takes a different return,
		// if not, fail
		{
			output2 = service.method2();
			System.out.println(output2);
		} // end-if
		
		assertFalse(output1.equals(output2));
	}
	
	public void testCacheClear() {
		// 
		IServiceToCache service = (IServiceToCache) ctx.getBean("serviceToCache");
		String output1, output2, output3, output4, output5, output6;
		
		// Call the method1ToCache(..), this method id in cache...
		// verify if the call to the service takes a different return,
		// if not, fail
		{
			output1 = service.method1ToCache("zut");
			System.out.println("output1 = " + output1);
			output2 = service.method1ToCache("zut");
			System.out.println("output2 = " + output2);
			output3 = service.method1ToCache("zut");
			System.out.println("output3 = " + output3);
			//
			assertTrue(output1.equals(output2));
			assertTrue(output1.equals(output3));
		} // end-if
		
		// Clear the cache
		{
			CacheInterceptor ix = (CacheInterceptor ) ctx.getBean("cacheInterceptor");
			ix.clear();
		}
		
		// Call the method1ToCache(..), this method id in cache...
		// verify if the call to the service takes a different return,
		// if not, fail
		{
			output4 = service.method1ToCache("tac");
			System.out.println("output4 = " + output4);
			output5 =  service.method1ToCache("zut");
			System.out.println("output5 = " + output5);
			output6 =  service.method1ToCache("zut");
			System.out.println("output6 = " + output6);
			//
			assertFalse(output4.equals(output5));
			assertTrue(output5.equals(output6));
		}
	}

	// -------------------------------------------------------- PRIVATE METHODS
	// ------------------------------------------------------ PROTECTED METHODS
	
	/**
	 * Method setUp
	 * @throws Exception
	 * @see junit.framework.TestCase#setUp()
	 */
	@Override
	protected void setUp() throws Exception {
		super.setUp();
		
		if(this.ctx == null){
			this.ctx = new FileSystemXmlApplicationContext("C:\\MyFiles\\Development\\Java\\dev\\test_AOP_cache\\WebContent\\WEB-INF\\spring-cacheinterceptor-config.xml");
		}
	}

	// --------------------------------------------- ADVANCED GETTERS & SETTERS
	// ------------------------------------------------------ GETTERS & SETTERS
}