package com.ho.test.aop.cache.service.common;

import java.lang.annotation.ElementType;
import java.util.Calendar;
import java.util.Enumeration;
import java.util.Hashtable;

/**
 * <p>Title: Cache</p>
 * <p>Copyright: Copyright (c) 2010</p>
 * <p>Company: HUO</p>
 * 
 * <p>
 * This class cache the given element.
 * </p>
 * 
 * @author Huseyin OZVEREN
 */
public class Cache <KeyType, ElementType> implements ICache<KeyType, ElementType> {
	
	// -------------------------------------------------------------- CONSTANTS
	// ----------------------------------------------------- PRIVATE ATTRIBUTES
	/**
	 * Contains the cached items.
	 */
	private Hashtable<KeyType, Pair<ElementType, Long>> cache = new Hashtable<KeyType, Pair<ElementType, Long>>();

	// ------------------------------------------------------------- EXCEPTIONS
	// ------------------------------------------------------- INTERNAL CLASSES
	// ----------------------------------------------------------- CONSTRUCTORS
	// --------------------------------------------------------- PUBLIC METHODS
	/**
	 * Put an item in the cache.
	 */
	public void put(KeyType key, ElementType value) {
		Pair<ElementType, Long> p = new Pair(value, Calendar.getInstance().getTimeInMillis()/1000);
		cache.put(key, p);
	}
	
	/**
	 * Retrieve an item from the Cache.
	 * @param key
	 */
	public ElementType get(KeyType key, long cacheTimeInSeconds) {
		Pair<ElementType, Long> p = cache.get(key);

		if (null == p) {
			return null;
		} // end-if
		
		if (isRefreshNeeded(p, cacheTimeInSeconds)) {
			remove(key);
			return null;
		} // end-if
		
		return p.getFirst();
	}
	
	/**
	 * Update an item in the cache.
	 */
	public void update(KeyType key, ElementType value) {
		Pair<ElementType, Long> p = new Pair(value, Calendar.getInstance().getTimeInMillis()/1000);
		cache.put(key, p);
	}
	
	/**
	 * Clear the cache.
	 */
	public void clear() {
		cache.clear();
	}

	/**
	 * Remove an item from the cache.
	 */
	public void remove(KeyType key) {
		cache.remove(key);
	}

	public String getCacheKeysCore4MBEANTestOnly(){
		if(cache != null){
			Enumeration<KeyType> enumKeys = cache.keys();
			if(enumKeys!=null){
				StringBuffer stbf = new StringBuffer("--- NEW KEY ---");
				while(enumKeys.hasMoreElements()){
					KeyType key = enumKeys.nextElement();
					stbf.append(key.toString()).append("--- NEW KEY ---");
				}
				return stbf.toString();
			}
		}
		return null;
	}
	
	/**
	 * @return true if a cache refresh is needed for the given element.
	 */
	public boolean isRefreshNeeded(Pair<ElementType, Long> p, long cacheTimeInSeconds) {
		
		// Verify if the item time is not greater than the given cacheTimeInSeconds
		boolean needRefresh = false;
		{
			// Compute the current item age
			long ageInSeconds = (Calendar.getInstance().getTimeInMillis()/1000) - p.getSecond();
			if (ageInSeconds>cacheTimeInSeconds) {
				 needRefresh = true;
			} // end-if
		} // end-block
		
		return needRefresh;

	}

	// -------------------------------------------------------- PRIVATE METHODS
	// ------------------------------------------------------ PROTECTED METHODS
	// --------------------------------------------- ADVANCED GETTERS & SETTERS
	// ------------------------------------------------------ GETTERS & SETTERS
}