package com.ho.test.aop.cache.service.common;

/**
 * <p>Title: ICache</p>
 * <p>Copyright: Copyright (c) 2010</p>
 * <p>Company: HUO</p>
 * 
 * <p>
 * Implementors define a caching algorithm.
 * <br/><b>All implementors must be threadsafe.</b> 
 * </p> 
 * 
 * @author Huseyin OZVEREN
 */
public interface ICache <KeyType, ElementType> {

	// -------------------------------------------------------------- CONSTANTS
	// ----------------------------------------------------- PRIVATE ATTRIBUTES
	// ------------------------------------------------------------- EXCEPTIONS
	// ------------------------------------------------------- INTERNAL CLASSES
	// ----------------------------------------------------------- CONSTRUCTORS
	// --------------------------------------------------------- PUBLIC METHODS
	
	/**
	 * Put an item in the cache.
	 */
	public void put(KeyType key, ElementType value);
	
	/**
	 * Retrieve an item from the Cache.
	 * <br/>If the cached item is older than the given cacheTimeInSeconds, remove it from the cache and return null.
	 * @param key
	 */
	public ElementType get(KeyType key, long cacheTimeInSeconds);
	
	/**
	 * Update an item in the cache.
	 */
	public void update(KeyType key, ElementType value);
	
	/**
	 * Clear the cache.
	 */
	public void clear();

	/**
	 * Remove an item from the cache.
	 */
	public void remove(KeyType key);

	// -------------------------------------------------------- PRIVATE METHODS
	// ------------------------------------------------------ PROTECTED METHODS
	// --------------------------------------------- ADVANCED GETTERS & SETTERS
	// ------------------------------------------------------ GETTERS & SETTERS
}