package com.ho.test.aop.cache.spring.utils;

import javax.servlet.ServletContext;

import org.springframework.web.context.support.XmlWebApplicationContext;


/**
 * Gives an access to Spring Container
 * <p>Title: ApplicationContextContainer</p>
 * <p>Copyright: Copyright (c) 2011</p>
 * <p>Company: HUO</p>
 * @author HUO
 */
public class ApplicationContextContainer {
	
    // ------------------------------------------------------------------ LOG4J
    // -------------------------------------------------------------- CONSTANTS
    private final static String SPRING_BUSINESS_CONFIG_XML = "/WEB-INF/spring-cacheinterceptor-config.xml";

    // ----------------------------------------------------- PRIVATE ATTRIBUTES
    /**
     * Instance
     */
    private static ApplicationContextContainer instance = null;
    
    /**
     * Contains the spring configuration.
     */
    private XmlWebApplicationContext ctx = null;
    
    // ------------------------------------------------------------- EXCEPTIONS
    // ------------------------------------------------------- INTERNAL CLASSES
    // ----------------------------------------------------------- CONSTRUCTORS
    private ApplicationContextContainer() {}

    // --------------------------------------------------------- PUBLIC METHODS
    /**
     * Getinstance method.
     */
    public static synchronized ApplicationContextContainer getInstance() {
        return getInstance(SPRING_BUSINESS_CONFIG_XML, null);
    }
    public static synchronized ApplicationContextContainer getInstance(String springContextFile, ServletContext servletContext) {
    	if (null == instance) {
    		instance = new ApplicationContextContainer();
    		instance.ctx = new XmlWebApplicationContext();  
    		instance.ctx.setConfigLocation(springContextFile);
    		instance.ctx.setServletContext(servletContext);
    		instance.ctx.refresh();
        } // end-if
        return instance;
    }
 
    
    /**
     * Retrieve the spring bean corresponding to the given key.
     * @param key
     * @return
     */
    public static Object getBean(String key) {
        return getInstance().ctx.getBean(key);
    }
    
    public XmlWebApplicationContext getContext() {
        return ctx;
    }
    
    /**
     * <p>
     * 		Cette methode permet de savoir si le contexte spring a dejaete charge (via le servletListener SpringContextLoader).
     * 		Ceci pour eviter de demarrer des connecteurs de communication alors que le deploiement de l'application n'est pas termine.
     * 		<br/>
     * 		<br/>Si les connecteurs de communication ne sont pas mis en attente al'aide de cette methode, 
     * 		<br/>ils risquent d'instancier un context spring qui provoquera une exception JTA au demarrage (probablement
     * 		<br/>liee au class loader unifie):
     * 			Error creating bean with name 'methodNamesTxInterceptor' defined in class path resource [spring-business-config.xml]: 
     * 			Cannot resolve reference to bean 'transactionManager' while setting bean property 'transactionManager'
     * </p>
     * @return
     */
    public static boolean isContextLoaded() {
    	return null!=instance && null!=instance.ctx;
    }

    // -------------------------------------------------------- PRIVATE METHODS
    // ------------------------------------------------------ PROTECTED METHODS
    // --------------------------------------------- ADVANCED GETTERS & SETTERS
    // ------------------------------------------------------ GETTERS & SETTERS
}
