package com.ho.test.mbean.common;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.management.Attribute;
import javax.management.AttributeList;
import javax.management.AttributeNotFoundException;
import javax.management.InstanceNotFoundException;
import javax.management.IntrospectionException;
import javax.management.InvalidAttributeValueException;
import javax.management.MBeanAttributeInfo;
import javax.management.MBeanException;
import javax.management.MBeanInfo;
import javax.management.MBeanOperationInfo;
import javax.management.MBeanParameterInfo;
import javax.management.MBeanServer;
import javax.management.MBeanServerFactory;
import javax.management.MalformedObjectNameException;
import javax.management.ObjectInstance;
import javax.management.ObjectName;
import javax.management.ReflectionException;


public class MbeanTools implements IRemoteMbeanService {

	/**
	 * Build the form out of a given mbean Name
	 * 
	 * @param mbeanForm
	 * @param name
	 * @throws MalformedObjectNameException
	 * @throws NullPointerException
	 * @throws InstanceNotFoundException
	 * @throws IntrospectionException
	 * @throws ReflectionException
	 */
	public MBeanVo buildMbeanVo(String name) throws MalformedObjectNameException, NullPointerException, InstanceNotFoundException, IntrospectionException, ReflectionException {
		MBeanVo vo = new MBeanVo();
		vo.setName(name);
		MBeanServer server = getServer();
		ObjectName objName = new ObjectName(name);
		Map<MBeanAttributeInfo, Object> map = getAttributeValues(objName);

		MBeanInfo info = server.getMBeanInfo(objName);
		vo.setAttributes(map);
		vo.setOperations(Arrays.asList(info.getOperations()));
		return vo;
	}

	public MBeanVo updateAttributes(String name, String[] attributeValues, String[] attributeKeys) throws MalformedObjectNameException, NullPointerException, InstanceNotFoundException, IntrospectionException, ReflectionException, ClassNotFoundException, SecurityException, NoSuchMethodException, IllegalArgumentException, IllegalAccessException, InvocationTargetException, AttributeNotFoundException, InvalidAttributeValueException, MBeanException {
		MBeanServer server = getServer();
		String type = null;
		Method valueOf = null;
		ObjectName objectName = new ObjectName(name);
		Map<MBeanAttributeInfo, Object> map = getAttributeValues(objectName);
		Collection<MBeanAttributeInfo> keys = map.keySet();
		for (MBeanAttributeInfo beanAttributeInfo : keys) {
			boolean found = false;
			String valueToUse = null;
			for (int i = 0; i < attributeKeys.length && !found; i++) {
				found = attributeKeys[i].equals(beanAttributeInfo.getName());
				valueToUse = attributeValues[i];
			}
			if (found) {
				type = beanAttributeInfo.getType();
				Class<?> clazz = getType(type);
				Object value = valueToUse.toString();
				if (!clazz.equals(String.class)) {
					valueOf = clazz.getMethod("valueOf", String.class);
					value = valueOf.invoke(null, valueToUse);
				}
				objectName.setMBeanServer(server);
				server.setAttribute(objectName, new Attribute(beanAttributeInfo.getName(), value));
			}
		}
		MBeanVo vo = buildMbeanVo(name);
		return vo;
	}

	/**
	 * Retrieves all the params value for by method names
	 * 
	 * @param paramValues
	 * @param NonVoidMethodName
	 * @param operations
	 * @return
	 */
	private Map<String, List<String>> getParamsValueByMethods(String[] paramValues, String[] NonVoidMethodName, MBeanOperationInfo[] operations) {
		Map<String, List<String>> map = new HashMap<String, List<String>>();

		int cptValues = 0;
		if (NonVoidMethodName != null) {
			for (int i = 0; i < NonVoidMethodName.length; i++) {
				String methodName = NonVoidMethodName[i];
				boolean found = false;
				MBeanOperationInfo beanOperationInfo = null;
				for (int j = 0; j < operations.length && !found; j++) {
					beanOperationInfo = operations[j];
					found = beanOperationInfo.getName().equals(methodName);
				}
				if (found) {
					int nbPArams = beanOperationInfo.getSignature().length;
					int countToReach = cptValues + nbPArams;
					ArrayList<String> list = new ArrayList<String>();
					map.put(methodName, list);
					while (countToReach > cptValues) {
						list.add(paramValues[cptValues]);
						cptValues++;
					}
				}

			}
		}
		return map;
	}

	/**
	 * Call an Mbean method
	 * 
	 * @param mapping
	 * @param form
	 * @param request
	 * @param response
	 * @return
	 * @throws NullPointerException
	 * @throws MalformedObjectNameException
	 * @throws ReflectionException
	 * @throws IntrospectionException
	 * @throws InstanceNotFoundException
	 * @throws ClassNotFoundException
	 * @throws NoSuchMethodException
	 * @throws SecurityException
	 * @throws InvocationTargetException
	 * @throws IllegalAccessException
	 * @throws IllegalArgumentException
	 * @throws MBeanException
	 */
	public MBeanVo invoke(String name, String methodName, String[] paramValues, String[] NonVoidMethodName) throws MalformedObjectNameException, NullPointerException, InstanceNotFoundException, IntrospectionException, ReflectionException, ClassNotFoundException, SecurityException, NoSuchMethodException, IllegalArgumentException, IllegalAccessException, InvocationTargetException, MBeanException {

		String message = "";
		String exeption = null;

		Collection<Object> params = new ArrayList<Object>();
		Collection<String> signatures = new ArrayList<String>();
		MBeanVo vo = null;
		if (methodName != null) {
			String type = null;
			Method valueOf = null;
			MBeanServer server = getServer();
			String currentBeanName = name;
			ObjectName objName = new ObjectName(name);
			MBeanInfo info = server.getMBeanInfo(objName);
			MBeanOperationInfo[] operations = info.getOperations();
			Map<String, List<String>> paramsForMethod = getParamsValueByMethods(paramValues, NonVoidMethodName, operations);
			boolean found = false;
			MBeanOperationInfo beanOperationInfo = null;
			for (int i = 0; i < operations.length && !found; i++) {
				beanOperationInfo = operations[i];
				found = beanOperationInfo.getName().equals(methodName);
			}

			if (found) {
				MBeanParameterInfo[] paramsInfo = beanOperationInfo.getSignature();
				List<String> listOfParamsValues = paramsForMethod.get(methodName);
				for (int i = 0; i < paramsInfo.length; i++) {
					MBeanParameterInfo beanParameterInfo = paramsInfo[i];
					type = beanParameterInfo.getType();
					signatures.add(type);
					Class<?> clazz = getType(type);
					String valueToUse = listOfParamsValues.get(i);
					Object value = valueToUse.toString();
					if (!clazz.equals(String.class)) {
						valueOf = clazz.getMethod("valueOf", String.class);
						value = valueOf.invoke(null, valueToUse);
					}
					params.add(value);
				}

				Object obj = server.invoke(objName, methodName, params.toArray(), signatures.toArray(new String[signatures.size()]));
				vo = buildMbeanVo(currentBeanName);
				vo.setPostInvokeMethod("METHOD: " + methodName + " PARAM(S): " + (params.isEmpty() ? "empty" : toStrList(params, ",")));
				if (obj != null) {
					message = obj.toString();

				}
			}

		}

		vo.setReturnValue(message);
		if (exeption != null) {
			vo.setReturnException(exeption);
		}

		return vo;
	}

	public Class<?> getType(String name) throws ClassNotFoundException {
		if (name.equals("byte"))
			return Byte.class;
		if (name.equals("short"))
			return Short.class;
		if (name.equals("int"))
			return Integer.class;
		if (name.equals("long"))
			return Long.class;
		if (name.equals("char"))
			return Character.class;
		if (name.equals("float"))
			return Float.class;
		if (name.equals("double"))
			return Double.class;
		if (name.equals("boolean"))
			return Boolean.class;
		if (name.equals("void"))
			return void.class;
		return Class.forName(name);
	}

	/**
	 * REtrieve all the mbeans matching the given pattern see ObjectName class
	 * for pattern info
	 * 
	 * @param mbeanPattern
	 * @return null if error occured (other than malformed mbeanPattern) or not
	 *         found
	 * @throws NullPointerException
	 * @throws MalformedObjectNameException
	 */
	public Map<ObjectName, ObjectInstance> getMbeanByPattern(String... mbeanPattern) throws MalformedObjectNameException, NullPointerException {
		MBeanServer server = this.getServer();
		Map<ObjectName, ObjectInstance> fullNames = null;
		Collection<ObjectName> names = new ArrayList<ObjectName>();
		for (String objectNameStr : mbeanPattern) {
			names.add(new ObjectName(objectNameStr));
		}
		try {
			fullNames = new HashMap<ObjectName, ObjectInstance>();
			for (ObjectName objectName : names) {
				Collection<ObjectName> listSpecificNames = server.queryNames(objectName, null);
				for (ObjectName currentName : listSpecificNames) {
					fullNames.put(currentName, server.getObjectInstance(currentName));
				}
			}
		} catch (Throwable th) {
			th.printStackTrace();
		}
		return fullNames;
	}

	public Collection<MBeanOperationInfo> listOperations(ObjectName name) {
		MBeanServer server = this.getServer();
		Collection<MBeanOperationInfo> methods = null;

		try {
			MBeanInfo info = server.getMBeanInfo(name);
			ObjectInstance instance = server.getObjectInstance(name);
			MBeanOperationInfo[] operations = info.getOperations();
			methods.addAll(Arrays.asList(operations));

		} catch (Throwable th) {
			th.printStackTrace();
		}

		return methods;
	}

	private MBeanServer getServer() {
		return (MBeanServer) MBeanServerFactory.findMBeanServer(null).iterator().next();
	}

	private boolean isManagedType(MBeanAttributeInfo attInfo) {
		boolean value = false;
		value = (String.class.getCanonicalName().equals(attInfo.getType()) || Boolean.class.getCanonicalName().equals(attInfo.getType()) || Integer.class.getCanonicalName().equals(attInfo.getType()) || Long.class.getCanonicalName().equals(attInfo.getType()));

		return value;

	}

	public Map<MBeanAttributeInfo, Object> getAttributeValues(ObjectName objName) throws InstanceNotFoundException, IntrospectionException, ReflectionException {
		Map<MBeanAttributeInfo, Object> map = new HashMap<MBeanAttributeInfo, Object>();
		MBeanServer server = this.getServer();
		MBeanInfo info = server.getMBeanInfo(objName);
		MBeanAttributeInfo[] attributesInfo = info.getAttributes();
		Map<String, MBeanAttributeInfo> wkdMap = new HashMap<String, MBeanAttributeInfo>();
		for (int i = 0; i < attributesInfo.length; i++) {
			MBeanAttributeInfo attInfo = attributesInfo[i];
			if (attInfo.isReadable()) {
				if (isManagedType(attInfo)) {
					wkdMap.put(attInfo.getName(), attInfo);
				}
			}
		}
		AttributeList list = server.getAttributes(objName, wkdMap.keySet().toArray(new String[wkdMap.size()]));
		for (Object object : list) {
			Attribute attr = (Attribute) object;
			map.put(wkdMap.get(attr.getName()), attr.getValue());
		}
		return map;
	}

	
    /**
     * Returns a String list of objects separated by the given separator and wrap the values withe the given wrapper
     * @param list
     * @param separator
     * @return null if list or separator is null
     */
     public static String toStrList(Collection<?> list,String separator) {
            String output=null;
            if(list!=null && separator!=null) {
                output=list.toString().replaceAll("\\[", "").replaceAll("\\]","");
                output=output.replaceAll("\\, ", separator);
            }
            return output;
    }
}
